AzCastBarDropDowns = {};
local cfg = AzCastBar_Config;

-- Shared Media
local LSM = LibStub and LibStub("LibSharedMedia-3.0",1);

-- DropDown Lists
AzCastBarDropDowns.FontFlags = {
	["|cffffa0a0None"] = "",
	["Outline"] = "OUTLINE",
	["Thick Outline"] = "THICKOUTLINE",
};

--------------------------------------------------------------------------------------------------------
--                                        Default DropDown Init                                       --
--------------------------------------------------------------------------------------------------------

local function Default_SelectValue(dropDown,entry,index)
	cfg[AzCastBarOptions.activeBar.token][dropDown.option.var] = entry.value;
	AzCastBar.ApplyBarSettings(AzCastBarOptions.activeBar);
end

function AzCastBarDropDowns.Default(dropDown,list)
	dropDown.SelectValueFunc = Default_SelectValue;
	for text, option in next, dropDown.option.list do
		list[#list + 1] = { text = text, value = option };
	end
end

--------------------------------------------------------------------------------------------------------
--                                          Lib Shared Media                                          --
--------------------------------------------------------------------------------------------------------

if (LSM) then
	LSM:Register("border","Solid","Interface\\Buttons\\WHITE8X8");
end

local LibSharedMediaSubstitute = not LSM and {
	["font"] = {
		["Friz Quadrata TT"] = "Fonts\\FRIZQT__.TTF",
		["Arial Narrow"] = "Fonts\\ARIALN.TTF",
		["Skurri"] = "Fonts\\SKURRI.TTF",
		["Morpheus"] = "Fonts\\MORPHEUS.TTF",
	},
	["background"] = {
		["Blizzard Tooltip"] = "Interface\\Tooltips\\UI-Tooltip-Background",
		["Solid"] = "Interface\\Buttons\\WHITE8X8",
	},
	["border"] = {
		["None"] = "Interface\\None",
		["Blizzard Dialog"]  = "Interface\\DialogFrame\\UI-DialogBox-Border",
		["Blizzard Tooltip"] = "Interface\\Tooltips\\UI-Tooltip-Border",
		["Solid"] = "Interface\\Buttons\\WHITE8X8",
	},
	["statusbar"] = {
		["Blizzard StatusBar"] = "Interface\\TargetingFrame\\UI-StatusBar",
	},
} or nil;

do
	local AZCB_Textures = {
		"HorizontalFade",
		"Pale",
		"Lines",
		"SmoothBar",
		"Streamline",
		"Streamline-Inverted",
		"Waterline",
	};
	local texturePath;
	local root = "Interface\\Addons\\AzCastBar\\Textures\\";
	for _, texture in ipairs(AZCB_Textures) do
		texturePath = root..texture;
		if (LSM) then
			LSM:Register("statusbar","|cffffff00"..texture,texturePath);
		else
			LibSharedMediaSubstitute["statusbar"][texture] = texturePath;
		end
	end
end

function AzCastBarDropDowns.SharedMediaLib(dropDown,list)
	local query = dropDown.option.media;
	dropDown.SelectValueFunc = Default_SelectValue;
	if (LSM) then
		for _, name in next, LSM:List(query) do
			list[#list + 1] = { text = name, value = LSM:Fetch(query,name) };
		end
	else
		for name, value in next, LibSharedMediaSubstitute[query] do
			list[#list + 1] = { text = name, value = value };
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                            List Plugins                                            --
--------------------------------------------------------------------------------------------------------

local function CopyPluginSettings_SelectValue(dropDown,entry,index)
	local cfgTo = cfg[AzCastBarOptions.activeBar.token];
	local cfgFrom = entry.value;
	-- using rawget here makes sure it goes default if the bar we copy from doesn't have this option set
	for option, value in next, AzCastBar.defOptions do
		cfgTo[option] = rawget(cfgFrom,option);
	end
	AzCastBar.ApplyBarSettings(AzCastBarOptions.activeBar);
	dropDown.label:SetText("|cff80ff80Settings Copied");
end

function AzCastBarDropDowns.CopyPluginSettings_Init(dropDown,list)
	dropDown.SelectValueFunc = CopyPluginSettings_SelectValue;
	for index, plugin in ipairs(AzCastBar.frames) do
		if (plugin ~= AzCastBarOptions.activeBar) then
			list[#list + 1] = { text = plugin.token, value = plugin.cfg, checked = false };
		end
	end
	dropDown.label:SetText("|cff00ff00Select Plugin...");
end

--------------------------------------------------------------------------------------------------------
--                                            List Layouts                                            --
--------------------------------------------------------------------------------------------------------

local layouts = {
	["Default"] = {
		enabled = true,
		showLabel = true,
		showTime = true,
		showTotalTime = false,
		swapLabels = false,
		textAlign = "LEFT",
		fadeTime = 0.6,

		alpha = 1,
		texture = "Interface\\Addons\\AzCastBar\\Textures\\Waterline",
		useSameBGTexture = false,
		colBackGround = { 0.3, 0.3, 0.3, 0.6 },
		backdropBG = "Interface\\Tooltips\\UI-Tooltip-Background",
		backdropIndent = -2.5,
		colBackdrop = { 0.1, 0.22, 0.35, 1.0 },

		strata = "MEDIUM",
		width = 250,
		height = 18,

		anchorPoint = "BOTTOM",
		anchorOffset = 6,
		iconAnchor = "LEFT",
		iconOffset = 2.5,
		hideIconBorder = true,

		fontFace = "Fonts\\FRIZQT__.TTF",
		fontFlags = "",
		fontSize = 12,
		colFont = { 1.0, 1.0, 1.0, 1.0 },
	},
	["Auras"] = {
		backdropIndent = 0,

		width = 200,
		height = 16,

		anchorPoint = "BOTTOM",
		anchorOffset = 0,
		iconAnchor = "LEFT",
		iconOffset = 0,
		hideIconBorder = true,

		fontSize = 10,
	},
};

local function LayoutTemplate_SelectValue(dropDown,entry,index)
	local barCfg = cfg[AzCastBarOptions.activeBar.token];
	for option, value in next, layouts[entry.value] do
		barCfg[option] = value;
	end
	AzCastBar.ApplyBarSettings(AzCastBarOptions.activeBar);
	dropDown.label:SetText("|cff80ff80Layout Loaded");
end

function AzCastBarDropDowns.LayoutTemplate_Init(dropDown,list)
	dropDown.SelectValueFunc = LayoutTemplate_SelectValue;
	for layoutName in next, layouts do
		list[#list + 1] = { text = layoutName, value = layoutName };
	end
	dropDown.label:SetText("|cff00ff00Pick Layout...");
end